@@ -7,10 +7,16 @@ use base qw/Test::HTTP::Server::Simple HTTP::Server::Simple::CGI/;
 
 use Time::HiRes qw(sleep time);
 use Data::Dumper;
+use Test::More;
 use LWP::UserAgent;
+use Net::EmptyPort ();
 
 sub new {
     my ($class, $port) = @_;
+
+    if (!$port) {
+        $port = Net::EmptyPort::empty_port();
+    }
     
     # Require a port parameter to be passed in.
     # Any default here would mean the tests don't run properly in parallel.
@@ -40,9 +46,6 @@ sub handle_request {
     # trickle tests.
     $self->stdout_handle->autoflush(1);
 
-     # warn "START REQUEST - " . time;
-     # warn Dumper($params);
-
     # Do the right thing depending on what is asked of us.
     if ( exists $params->{redirect} ) {
         my $num = $params->{redirect} || 0;
@@ -81,6 +84,15 @@ sub handle_request {
         print "Trickled for '$trickle_for'.\n";
     }
 
+    elsif ( exists $params->{cookie} ) {
+        print $cgi->header(
+            -nph => 1,
+            -cookie => $cgi->cookie(-name => "x", value => "test"),
+        );
+
+        print "Sent test cookie\n";
+    }
+
     elsif ( exists $params->{bad_header} ) {
         my $headers = $cgi->header( -nph => 1, );
 
@@ -153,4 +165,39 @@ sub act_as_proxy {
     return 1;
 }
 
+# To allow act_as_proxy to work with HTTP::Server::Simple::CGI versions above
+# 0.41_1, where better support for RFC1616 was added, we have to override the
+# parse_request() method to match the pre-0.45_1 version of the method. Lame
+# and hacky but it works.
+sub parse_request {
+    my $self = shift;
+    my $chunk;
+    while ( sysread( STDIN, my $buff, 1 ) ) {
+        last if $buff eq "\n";
+        $chunk .= $buff;
+    }
+    defined($chunk) or return;
+    $_ = $chunk;
+
+    m/^(\w+)\s+(\S+)(?:\s+(\S+))?\r?$/;
+    my $method   = $1 || '';
+    my $uri      = $2 || '';
+    my $protocol = $3 || '';
+
+    return ( $method, $uri, $protocol );
+}
+
+# Change print() to note() in HTTP::Server::Simple::print_banner
+sub print_banner {
+    my $self = shift;
+
+    note(
+        ref($self)
+        . ": You can connect to your server at "
+        . "http://localhost:"
+        . $self->port
+        . "/"
+    );
+}
+
 1;
